==> sagua_0.2.2-1.tar.gz: sagua-0.2.2/include/playlist.h <==
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_playlist_h
#define GUARD_playlist_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file playlist.h
/// @brief Class for Playlist header file.

#include <string>
#include <cstring>
#include <queue>
#include <vector>

#include <gtkmm/treeview.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/liststore.h>
#include <gtkmm/menu.h>

#include "misc.h"

class MainWnd;
class Tag;
class TagFactory;

/// @brief Class for Playlist.
/// @todo Reduce relationship to MainWnd.
class Playlist : public Gtk::TreeView
{
public:
    Playlist(MainWnd& mainWnd, const std::string& listId, const unsigned int& titleWidth = 470, const bool& loadlist = true);
    virtual ~Playlist();

    void AddFile(const std::string& filePath, const bool& addListFile);
    void AddFiles(const std::vector<std::string>& files, const bool& addListFile);
    void AddDirectory(const std::string& directoryPath, const bool& recursive);

    /// @brief Remove all items.
    void RemoveAll() 
    {   
        OnMenuRemoveAll(); 
    }

    /// @brief Get file path.
    std::string Get()
    {
        return GetCurrentPositionFilename();
    }
    std::string GetNext();
    std::string GetPrev();
    std::string GetRandom();
    void ForceUpdateToppage();
    void ForceUpdatePage();

    void ShowCurrentFileInfoDlg() const;

private:
    class ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
    public:
        ModelColumns()
        {
            add(tagParsed_);
            add(id_);
            add(filename_);
            add(title_);
            add(time_);
            add(timeSeconds_);
        }

        Gtk::TreeModelColumn<bool> tagParsed_;
        Gtk::TreeModelColumn<unsigned int> id_;
        Gtk::TreeModelColumn<std::string> filename_;
        Gtk::TreeModelColumn<Glib::ustring> title_;
        Gtk::TreeModelColumn<Glib::ustring> time_;
        Gtk::TreeModelColumn<unsigned int> timeSeconds_;
    };

    class EachRow 
    {
    public:
        EachRow(const Gtk::TreeModel::iterator& iter, const Glib::ustring& title, const Glib::ustring& time, const unsigned int& timeSeconds)
            : iter_(iter), title_(title), time_(time), timeSeconds_(timeSeconds)
        {}

        Gtk::TreeModel::iterator iter_;
        Glib::ustring title_;
        Glib::ustring time_;
        unsigned int timeSeconds_;
    };

    ModelColumns columns_;

    Gtk::Menu menuPopup_;
    Gtk::TreeModel::iterator pos_;

    Glib::Mutex tagParsingMutex_;
    Glib::Dispatcher signalRowToFill_;

    std::queue<EachRow> rowQueue_;
    std::vector<std::string> listType_;

    const std::string fileProtocol_;		///< String to determine file protocol, "file://"
    const std::string listId_;			///< Identification string of list.

    const unsigned int mainWndPanelHeight_;
    const unsigned int eachRowHeight_;
    unsigned int count_;				///< Number of added items.
    unsigned int tagProcessCount_;			///< Number of items reading tag at one thread.

    bool inTagParsing_;				///< Tag parsing thread is running.
    bool stopTagParsing_;				///< To stop tag parsing.

    Glib::RefPtr<Gtk::ListStore> treeModel_;

    const std::auto_ptr<TagFactory> tagFactory_;

    MainWnd& mainWnd_;

    Glib::Thread* tagParsingThread_;

    virtual bool on_button_press_event(GdkEventButton* ev);
    virtual void on_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column);
    virtual void on_set_scroll_adjustments(Gtk::Adjustment* hadj, Gtk::Adjustment* vadj);
    virtual void on_size_allocate(Gtk::Allocation& allocation);

    void FillRowFromQueue(); 

    std::string GetCurrentPositionFilename(); 
    void ParseTag(Gtk::TreeModel::iterator& iter, const bool& thread);
    void GetInfoFromTag(Gtk::TreeModel::iterator& iter);

    void ShowFileInfoDlg(const std::string& filename) const;

    void OnDragDataReceived(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selectionData, guint info, guint time);

    void OnMenuRemove();
    void OnMenuRemoveAll();
    void OnMenuSortFilename();
    void OnMenuInfo();

    void OnVScrolled();

    void LoadLastList();

    void LoadList(const std::string& filename);
    void SaveList();

    void StopThread();

    void CreatePopupMenuList();
    void RegisterTagFactory();

    void AddDragedFiles(const std::string& dragedString);

    std::string NullFilename();

    /// @brief Filename compare function to sort list.
    int FilenameCompare(const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b)
    {
        std::string strA = (*a)[columns_.filename_];
        std::string strB = (*b)[columns_.filename_];

        std::string aPath = Glib::path_get_dirname(strA);
        std::string bPath = Glib::path_get_dirname(strB);

        // If files in same directory, sort them
        // or sort for directory.	
        int rtn = std::strncmp(aPath.c_str(), bPath.c_str(), aPath.size());
        if(rtn == 0)
            return std::strncmp(strA.c_str(), strB.c_str(), strA.size());
        else	
            return rtn;
    }

    /// @brief Move cursor to coresspond iterator and scroll to that.
    void MoveCursorAndScroll(const Gtk::TreeModel::iterator& iter)
    {
        Gtk::TreeModel::Path path;
        if(iter) {
            path = Gtk::TreeModel::Path(iter);
        } else {
            path = Gtk::TreeModel::Path("0");
        }
        set_cursor(path);
        scroll_to_row(path, 0.5);
    }

    /// @brief If list is empty, return true.
    bool IsListEmpty()
    {
        if(count_ == 0 || !treeModel_->children().begin())
            return true;

        return false;
    }

    /// @brief If stored iterator is last row, return true.
    bool IsPositionLastRow() const
    {
        return pos_ == --(treeModel_->children().end());
    }

    /// @brief If storef iterator is first row, return true.
    bool IsPositionFirstRow() const
    {
        return pos_ == treeModel_->children().begin();
    }

    /// @brief Get next row item's filename.
    std::string GetNextRowFilename()
    {
        ++pos_;

        return GetCurrentPositionFilename();
    }

    /// @brief Get previous row item's filename.
    std::string GetPrevRowFilename()
    {
        pos_--;

        return GetCurrentPositionFilename();
    }

    /// @brief Get last row item's filename.
    std::string GetLastRowFilename()
    {
        pos_ = --(treeModel_->children().end());

        return GetCurrentPositionFilename();
    }

    /// @brief Calculate row number per page.
    void CalcTagProcessCount(const unsigned int& height)
    {
        tagProcessCount_ = height / eachRowHeight_ + 2;
    }

    /// @brief Fill row.
    void FillRow(Gtk::TreeModel::iterator& iter, const Glib::ustring& title, const Glib::ustring& totalTime, const unsigned long& timeSeconds)
    {
        (*iter)[columns_.title_] = title;
        (*iter)[columns_.time_] = totalTime;
        (*iter)[columns_.timeSeconds_] = timeSeconds;

        if(timeSeconds != 1)
            (*iter)[columns_.tagParsed_] = true;
    }

    /// @brief Get top position of treeview iterator.
    Gtk::TreeModel::iterator GetTopPositionIterator()
    {
        Gtk::TreePath eachPath;
        Gtk::TreeViewColumn* eachColumn = NULL;
        int cellX = 0;
        int cellY = 0;

        get_path_at_pos(0, 0, eachPath, eachColumn, cellX, cellY);

        return treeModel_->get_iter(eachPath);
    }

    /// @brief If string begin with '#', return true.
    bool IsComment(const std::string& line)
    {
        if(line[0] == '#')
            return true;

        return false;
    }

    /// @brief Elliminate last white space character.
    void TrimLastSpaceCharacter(std::string& dest)
    {
        const std::string::reverse_iterator space( find_if(dest.rbegin(), dest.rend(), std::not1(IsCharSpace())) );
        if(space != dest.rend() && space != dest.rbegin()) {
            dest = std::string(dest.begin(), space.base());

        }
    }

    /// @brief Get random number.
    ///
    /// Get random number that does not repeat twice.
    unsigned int GetRandomNumber()
    {
        // Prevent undefined behavior
        if(count_ == 1)
            return 1;

        Glib::Rand rand;

        if(!pos_)
            pos_ = treeModel_->children().begin();

        // I don't want get same element.
        const unsigned int previousNumber = (*pos_)[columns_.id_] - 1;
        unsigned int randomNumber = 0;
        do {

            randomNumber = rand.get_int_range(0, count_);

        } while(randomNumber == previousNumber);

        return randomNumber;
    }

    /// @brief Move stored iterator from begin to number.
    void MoveNthPosition(const unsigned int& number)
    {
        pos_ = treeModel_->children().begin();

        for(unsigned int i = 0; i != number; ++i)
            ++pos_;
    }

    /// @brief Register list type extension.
    void RegisterListType()
    {
        listType_.push_back("m3u");
        // sort container if there are many items, or use associative container.
    }

    /// @brief If filePath's extension is in listType_, return true.
    bool IsListType(const std::string& filePath)
    {
        return std::find(listType_.begin(), listType_.end(), GetExt(filePath)) != listType_.end();
    }

    /// @brief Set title column properties.
    void SetTitleColumnProperties(const unsigned int& titleWidth)
    {
        get_column(1)->set_resizable(true);
        get_column(1)->set_fixed_width(titleWidth);
        get_column(1)->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
    }

};

#endif

==> sagua_0.2.3-1.tar.gz: sagua-0.2.3/include/playlist.h <==
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_playlist_h
#define GUARD_playlist_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file playlist.h
/// @brief Class for Playlist header file.

#include <string>
#include <cstring>
#include <queue>
#include <vector>
#include <map>

#include <gtkmm/treeview.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/liststore.h>
#include <gtkmm/menu.h>

#include "misc.h"

class MainWnd;
class Tag;
class TagFactory;
class ListFactory;

/// @brief Class for Playlist.
/// @todo Reduce relationship to MainWnd.
class Playlist : public Gtk::TreeView
{
    friend class ListFile;

public:
    Playlist(MainWnd& mainWnd, const std::string& listId, const unsigned int& titleWidth = 470, const bool& loadlist = true);
    virtual ~Playlist();

    void AddFiles(const std::vector<std::string>& files, const bool& addListFile);

    void AddDirectory(const std::string& directoryPath, const bool& recursive);

#ifdef ENABLE_HTTP

    void AddUrl(const std::string& destUrl);

#endif

    /// @brief Remove all items.
    void RemoveAll() 
    {   
        OnMenuRemoveAll(); 
    }

    /// @brief Get file path.
    std::string Get()
    {
        return GetCurrentPositionFilename();
    }
    std::string GetNext();
    std::string GetPrev();
    std::string GetRandom();
    void ForceUpdateToppage();
    void ForceUpdatePage();

    Glib::ustring CurrentItemTitle()
    {
        return (*pos_)[columns_.title_];
    }

    Glib::ustring CurrentItemTime()
    {
        return (*pos_)[columns_.time_];
    }

    unsigned int CurrentItemTimeSeconds()
    {
        return (*pos_)[columns_.timeSeconds_];
    }

    void ShowCurrentFileInfoDlg() const;

private:
    class ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
    public:
        ModelColumns()
        {
            add(tagParsed_);
            add(id_);
            add(filename_);
            add(title_);
            add(time_);
            add(timeSeconds_);
        }

        Gtk::TreeModelColumn<bool> tagParsed_;
        Gtk::TreeModelColumn<unsigned int> id_;
        Gtk::TreeModelColumn<std::string> filename_;
        Gtk::TreeModelColumn<Glib::ustring> title_;
        Gtk::TreeModelColumn<Glib::ustring> time_;
        Gtk::TreeModelColumn<unsigned int> timeSeconds_;
    };

    class EachRow 
    {
    public:
        EachRow(const Gtk::TreeModel::iterator& iter, const Glib::ustring& title, const Glib::ustring& time, const unsigned int& timeSeconds)
            : iter_(iter), title_(title), time_(time), timeSeconds_(timeSeconds)
        {}

        Gtk::TreeModel::iterator iter_;
        Glib::ustring title_;
        Glib::ustring time_;
        unsigned int timeSeconds_;
    };

    ModelColumns columns_;

    Gtk::Menu menuPopup_;
    Gtk::TreeModel::iterator pos_;

    Glib::Mutex tagParsingMutex_;
    Glib::Dispatcher signalRowToFill_;

    std::queue<EachRow*> rowQueue_;
    std::vector<std::string> listType_;

    static const std::string fileProtocol_;		///< String to determine file protocol, "file://"
    const std::string listId_;			///< Identification string of list.

    static const unsigned int mainWndPanelHeight_;
    static const unsigned int eachRowHeight_;
    unsigned int count_;				///< Number of added items.
    unsigned int tagProcessCount_;			///< Number of items reading tag at one thread.

    bool inTagParsing_;				///< Tag parsing thread is running.
    bool stopTagParsing_;				///< To stop tag parsing.

    Glib::RefPtr<Gtk::ListStore> treeModel_;

    static const std::auto_ptr<TagFactory> tagFactory_;
    static const std::auto_ptr<ListFactory> listFactory_;

    MainWnd& mainWnd_;

    Glib::Thread* tagParsingThread_;

    virtual bool on_button_press_event(GdkEventButton* ev);
    virtual void on_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column);
    virtual void on_set_scroll_adjustments(Gtk::Adjustment* hadj, Gtk::Adjustment* vadj);
    virtual void on_size_allocate(Gtk::Allocation& allocation);

    void AddFile(const std::string& filePath, const bool& addListFile);
    void AddFile(const std::string& filePath)
    {
        Gtk::TreeModel::Row row( *(treeModel_->append()) );
        row[columns_.tagParsed_] = false;
        row[columns_.id_] = ++count_;
        row[columns_.filename_] = filePath;
        row[columns_.timeSeconds_] = 1;
    }

    void AddFile(const std::string& filePath,
                 const Glib::ustring& title,
                 const Glib::ustring& time,
                 const unsigned int& seconds)
    {
        Gtk::TreeModel::Row row( *(treeModel_->append()) );
        row[columns_.tagParsed_] = true;
        row[columns_.id_] = ++count_;
        row[columns_.filename_] = filePath;
        // need convert to utf-8?
        row[columns_.title_] = filePath;
        // convert from time seconds ?
        row[columns_.time_] = time;
        row[columns_.timeSeconds_] = seconds;
    }


    void FillRowFromQueue(); 

    std::string GetCurrentPositionFilename(); 
    void ParseTag(Gtk::TreeModel::iterator& iter, const bool& thread);
    void GetInfoFromTag(Gtk::TreeModel::iterator& iter);

    void ShowFileInfoDlg(const std::string& filename) const;

    void OnDragDataReceived(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selectionData, guint info, guint time);

    void OnMenuRemove();
    void OnMenuRemoveAll();
    void OnMenuSortFilename();
    void OnMenuInfo();

    void OnVScrolled();

    void LoadLastList();

#ifdef ENABLE_HTTP

    void GetListFromUrl(const std::string& destUrl,
                        const std::string& listFile);

#endif

    void LoadList(const std::string& filename);
    void SaveList();

    void StopThread();

    void CreatePopupMenuList();
    void RegisterTagFactory();

    void RegisterListType();
    bool IsListType(const std::string& filePath);

    void AddDragedFiles(const std::string& dragedString);

    std::string NullFilename();

    /// @brief Filename compare function to sort list.
    int FilenameCompare(const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b)
    {
        const std::string strA((*a)[columns_.filename_]);
        const std::string strB((*b)[columns_.filename_]);

        const std::string aPath(Glib::path_get_dirname(strA));
        const std::string bPath(Glib::path_get_dirname(strB));

        // If files in same directory, sort them
        // or sort for directory.	
        const int rtn = std::strncmp(aPath.c_str(), bPath.c_str(), aPath.size());
        if(rtn == 0)
            return std::strncmp(strA.c_str(), strB.c_str(), strA.size());
        else
            return rtn;
    }

    /// @brief Move cursor to coresspond iterator and scroll to that.
    void MoveCursorAndScroll(const Gtk::TreeModel::iterator& iter)
    {
        Gtk::TreeModel::Path path;

        if(iter) {

            path = Gtk::TreeModel::Path(iter);

        } else {

            path = Gtk::TreeModel::Path("0");

        }

        set_cursor(path);
        scroll_to_row(path, 0.5);
    }

    /// @brief If list is empty, return true.
    bool IsListEmpty()
    {
        if(count_ == 0 || !treeModel_->children().begin())
            return true;

        return false;
    }

    /// @brief If stored iterator is last row, return true.
    bool IsPositionLastRow() const
    {
        return pos_ == --(treeModel_->children().end());
    }

    /// @brief If storef iterator is first row, return true.
    bool IsPositionFirstRow() const
    {
        return pos_ == treeModel_->children().begin();
    }

    /// @brief Get next row item's filename.
    std::string GetNextRowFilename()
    {
        ++pos_;

        return GetCurrentPositionFilename();
    }

    /// @brief Get previous row item's filename.
    std::string GetPrevRowFilename()
    {
        pos_--;

        return GetCurrentPositionFilename();
    }

    /// @brief Get last row item's filename.
    std::string GetLastRowFilename()
    {
        pos_ = --(treeModel_->children().end());

        return GetCurrentPositionFilename();
    }

    /// @brief Calculate row number per page.
    void CalcTagProcessCount(const unsigned int& height)
    {
        tagProcessCount_ = height / eachRowHeight_ + 2;
    }

    /// @brief Fill row.
    void FillRow(Gtk::TreeModel::iterator& iter, const Glib::ustring& title, const Glib::ustring& totalTime, const unsigned long& timeSeconds)
    {
        (*iter)[columns_.title_] = title;
        (*iter)[columns_.time_] = totalTime;
        (*iter)[columns_.timeSeconds_] = timeSeconds;

        if(timeSeconds != 1)
            (*iter)[columns_.tagParsed_] = true;
    }

    /// @brief Get top position of treeview iterator.
    Gtk::TreeModel::iterator GetTopPositionIterator()
    {
        // useless variables
        Gtk::TreeViewColumn* eachColumn;
        int cellX = 0;
        int cellY = 0;

        Gtk::TreePath eachPath;
        get_path_at_pos(0, 0, eachPath, eachColumn, cellX, cellY);

        return treeModel_->get_iter(eachPath);
    }


    /// @brief Get random number.
    ///
    /// Get random number that does not repeat twice.
    unsigned int GetRandomNumber()
    {
        // Prevent undefined behavior
        if(count_ == 1)
            return 1;

        Glib::Rand rand;

        if(!pos_)
            pos_ = treeModel_->children().begin();

        // I don't want get same element.
        const unsigned int previousNumber = (*pos_)[columns_.id_] - 1;
        unsigned int randomNumber = 0;
        do {

            randomNumber = rand.get_int_range(0, count_);

        } while(randomNumber == previousNumber);

        return randomNumber;
    }

    /// @brief Move stored iterator from begin to number.
    void MoveNthPosition(const unsigned int& number)
    {
        pos_ = treeModel_->children().begin();

        for(unsigned int i = 0; i != number; ++i)
            ++pos_;
    }

    /// @brief Set title column properties.
    void SetTitleColumnProperties(const unsigned int& titleWidth)
    {
        get_column(1)->set_resizable(true);
        get_column(1)->set_fixed_width(titleWidth);
        get_column(1)->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
    }

    Gtk::TreeModel::iterator GetFirstIter() const
    {
        return treeModel_->children().begin();
    }

    void GetRawInformation(const Gtk::TreeModel::iterator& iter, unsigned int& seconds, Glib::ustring& title, std::string& filename) const
    {
        seconds = (*iter)[columns_.timeSeconds_];
        title = (*iter)[columns_.title_];
        filename = (*iter)[columns_.filename_];
    }
};

class ListFile
{
public:
    virtual void Read(const std::string& filename, Playlist& output) = 0;
    virtual void Write(const Playlist& input, const std::string& filename) = 0;

protected:
    void AddFile(const std::string& filePath, Playlist& playlist)
    {
        playlist.AddFile(filePath, false);
    }

#ifdef ENABLE_HTTP

    void AddUrl(const std::string& filePath, Playlist& playlist)
    {
        playlist.AddUrl(filePath);
    }

#endif

    Gtk::TreeModel::iterator GetFirstIter(const Playlist& playlist) const
    {
        return playlist.GetFirstIter();
    }

    void GetRawInformation(const Playlist& playlist, const Gtk::TreeModel::iterator& iter, unsigned int& seconds, Glib::ustring& title, std::string& filename) const
    {
        playlist.GetRawInformation(iter, seconds, title, filename);
    }

    /// @brief If string begin with '#', return true.
    bool IsComment(const std::string& line)
    {
        if(line[0] == '#')
            return true;

        return false;
    }

    /// @brief Elliminate last white space character.
    void TrimLastSpaceCharacter(std::string& dest)
    {
        const std::string::reverse_iterator space( find_if(dest.rbegin(), dest.rend(), std::not1(IsCharSpace())) );
        if(space != dest.rend() && space != dest.rbegin()) {
            dest = std::string(dest.begin(), space.base());

        }
    }
};

class M3UList : public ListFile
{
public:
    virtual void Read(const std::string& filename, Playlist& output);
    virtual void Write(const Playlist& input, const std::string& filename);
};

class PLSList : public ListFile
{
public:
    virtual void Read(const std::string& filename, Playlist& output);
    virtual void Write(const Playlist& input, const std::string& filename);
};

std::auto_ptr<ListFile> CreateM3UList();
std::auto_ptr<ListFile> CreatePLSList();

class ListFactory
{
public:
    /// @brief Register id to callback map.
    /// @param id Identification string 
    /// @param creator callback to produce object
    /// @return Success of register.
    bool Register(const std::string& id, std::auto_ptr<ListFile> (*creator)())
    {
        return callbacks_.insert( callback::value_type(id, creator) ).second;
    }

    /// @brief Unregister id from callback map.
    /// @param id Identification string.
    /// @return Success of unregister.
    bool Unregister(const std::string& id)
    {
        return callbacks_.erase(id) == 1;
    }

    /// @brief Check factory has filename's type.
    /// @param filename Filename to check.
    /// @return Return true if factory has filename's type.
    bool Exist(const std::string& filename) const
    {
        callback::const_iterator i = callbacks_.find(GetExt(filename));

        if(i != callbacks_.end())
            return true;
        else
            return false;	
    }

    /// @brief Create ListFile object.
    /// @param filename Filename to play.
    /// @return ListFile object.
    std::auto_ptr<ListFile> Create(const std::string& filename)
    {
        callback::const_iterator i = callbacks_.find(GetExt(filename));

        if(i != callbacks_.end())
            return (i->second)();
        else
            return std::auto_ptr<ListFile>();	
    }

    bool Empty() const
    {
        return callbacks_.empty();
    }

private:
    typedef std::map<std::string, std::auto_ptr<ListFile> (*)()> callback;
    callback callbacks_;	    ///< callback map
};

#endif
